<?php

/**
 * @file
 * Batch processing functions for admin pages.
 */

/**
 * Batch callback; Import affiliate store subscription.
 *
 * @param string $message
 *   Status message to show when running the batch. Available placeholders for
 *   used in message are @current and @total, which are currently processing
 *   item and total item count respectively.
 * @param array $options
 *   Options specified when submitting batch from form.
 */
function affiliate_store_batch_import_subscription($message, $options,
&$context) {
  affiliate_store_include('batch');
  $type = 'product:%';
  if (!isset($context['sandbox']['progress'])) {
    $context['sandbox']['progress'] = 0;
    $context['sandbox']['total'] = affiliate_store_batch_count($type);
    if (!$context['sandbox']['total']) {
      return;
    }
  }

  // Run as user 1 since the batch must has permission to create, update, or
  // delete nodes.
  affiliate_store_include('common');
  affiliate_store_impersonate_user(1);
  $context['max_operation'] = $options['update_throttle'];
  $result = affiliate_store_batch_dequeue($type, $context);
  affiliate_store_revert_user();

  if ($result === TRUE) {
    // Override original batch message with number of batches finished.
    $context['message'] = strtr(
      $message,
      array(
        '@current' => $context['sandbox']['progress'] + 1,
        '@total' => $context['sandbox']['total'],
      )
    );
    $context['sandbox']['progress'] += (int) $context['finished'];
  }
  elseif ($result === NULL) {
    return;
  }

  $context['finished'] = (
    $context['sandbox']['progress'] / $context['sandbox']['total']
  );
  sleep($options['sleep']);
}

/**
 * Batch callback; Fetch affiliate store subscriptions and perform update.
 *
 * @param array $options
 *   Options specified when submitting batch from form.
 */
function affiliate_store_batch_fetch_subscription($options, &$context) {
  affiliate_store_include('batch');
  $type = 'subscription:%';
  if (!isset($context['sandbox']['progress'])) {
    if (!affiliate_store_batch_count($type, TRUE)) {
      // Queue new update if no existing batch.
      affiliate_store_include('subscription');
      affiliate_store_subscription_sync($context['results']['errors']);
    }
    $context['sandbox']['progress'] = 0;
    $context['sandbox']['total'] = affiliate_store_batch_count($type);
    if (!$context['sandbox']['total']) {
      return;
    }
  }

  $context['max_operation'] = $options['update_throttle'];
  $result = affiliate_store_batch_dequeue($type, $context);
  if ($result === TRUE) {
    // Override original batch message with number of batches finished.
    $context['message'] = t(
      'Fetching subscriptions: @current of @total.',
      array(
        '@current' => $context['sandbox']['progress'] + 1,
        '@total' => $context['sandbox']['total'],
      )
    );
    $context['sandbox']['progress'] += (int) $context['finished'];
  }
  elseif ($result === NULL) {
    return;
  }

  $context['finished'] = (
    $context['sandbox']['progress'] / $context['sandbox']['total']
  );
  sleep($options['sleep']);
}

/**
 * Batch finish callback for affiliate store update process.
 */
function affiliate_store_batch_update_finish($success, $results, $operations) {
  // Remove duplicate product deleted messages set by node_delete().
  $messages = drupal_get_messages('status');
  if (!empty($messages['status'])) {
    foreach ($messages['status'] as $message) {
      if (strpos($message, t('Affiliate store product')) !== 0) {
        // Put back unrelated messages.
        drupal_set_message($message);
      }
    }
  }

  if ($success) {
    affiliate_store_include('batch');
    $errors = affiliate_store_batch_results($results, 'errors');
    if ($errors) {
      $message = t('Error occurred during update.');
      $message .= theme('item_list', $errors);
      drupal_set_message($message, 'error');
    }
    else {
      // Update affiliate store last update time so that cron knows that update
      // is run already.
      variable_set('affiliate_store_last_update', time());
    }

    $added = affiliate_store_batch_results($results, 'product:added', TRUE);
    if ($added) {
      $message = format_plural(
        count($added),
        '1 product has been added.',
        '@count products have been added.'
      );
      drupal_set_message($message);
    }
    $updated = affiliate_store_batch_results($results, 'product:updated', TRUE);
    if ($updated) {
      $message = format_plural(
        count($updated),
        '1 product has been updated.',
        '@count products have been updated.'
      );
      drupal_set_message($message);
    }
    $deleted = affiliate_store_batch_results($results, 'product:deleted', TRUE);
    if ($deleted) {
      $message = format_plural(
        count($deleted),
        '1 product has been deleted.',
        '@count products have been deleted.'
      );
      drupal_set_message($message);
    }

    $messages = affiliate_store_batch_results($results);
    if ($messages) {
      watchdog(
        'affiliate store',
        'Affiliate store manually updated. !result',
        array('!result' => theme('item_list', $messages))
      );
    }
    elseif (!$errors) {
      drupal_set_message(t('Affiliate store is up to date.'));
    }
  }
  else {
    $operation = reset($operations);
    $message = t(
      'An error occurred while processing %op with arguments: @args',
      array('%op' => $operation[0], '@args' => print_r($operation[1], TRUE))
    );
    drupal_set_message($message, 'error');
  }
}

/**
 * Batch callback; Rename existing term names using conversion case configured
 * at settings page.
 */
function affiliate_store_batch_rename_term_names(&$context) {
  if (!isset($context['sandbox']['progress'])) {
    $context['sandbox']['data'] = array();
    $categories = array('category', 'subcategory', 'subsubcategory');
    foreach (affiliate_store_vocabularies($categories) as $vocab => $info) {
      $result = db_query(
        "SELECT {$info['as']}tid AS tid, title FROM {affiliate_store_$vocab}"
      );
      while ($term = db_fetch_array($result)) {
        $context['sandbox']['data'][] = $term;
      }
    }
    $context['sandbox']['progress'] = 0;
    $context['sandbox']['total'] = count($context['sandbox']['data']);
    if (!$context['sandbox']['total']) {
      return;
    }
  }

  $context['message'] = t(
    'Renaming term names: @current of @total.',
    array(
      '@current' => $context['sandbox']['progress'] + 1,
      '@total' => $context['sandbox']['total'],
    )
  );

  affiliate_store_include('common');
  $term = current($context['sandbox']['data']);
  $name = affiliate_store_convert_term_case($term['title']);
  db_query(
    "UPDATE {term_data} SET name = '%s' WHERE tid = %d", $name, $term['tid']
  );
  array_shift($context['sandbox']['data']);

  $context['results'][] = t('Updated term name %name', array('%name' => $name));
  ++$context['sandbox']['progress'];

  $context['finished'] = (
    $context['sandbox']['data'] ?
    ($context['sandbox']['progress'] / $context['sandbox']['total']) : 1
  );
}

/**
 * Batch callback; Rename existing term synonyms using conversion case
 * configured at settings page.
 */
function affiliate_store_batch_rename_term_synonyms(&$context) {
  if (!isset($context['sandbox']['progress'])) {
    $context['sandbox']['data'] = array();
    $categories = array('category', 'subcategory', 'subsubcategory');
    foreach (affiliate_store_vocabularies($categories) as $vocab => $info) {
      $result = db_query(
        "SELECT ts.tid, ts.name FROM {term_synonym} AS ts
        JOIN {affiliate_store_$vocab} AS v ON v.{$info['as']}tid = ts.tid"
      );
      while ($synonym = db_fetch_array($result)) {
        $context['sandbox']['data'][] = $synonym;
      }
    }
    $context['sandbox']['progress'] = 0;
    $context['sandbox']['total'] = count($context['sandbox']['data']);
    if (!$context['sandbox']['total']) {
      return;
    }
  }

  $context['message'] = t(
    'Renaming term synonyms: @current of @total.',
    array(
      '@current' => $context['sandbox']['progress'] + 1,
      '@total' => $context['sandbox']['total'],
    )
  );

  affiliate_store_include('common');
  $synonym = current($context['sandbox']['data']);
  // Titlecase and smartcase require the term in lowercase 1st.
  $name = affiliate_store_convert_case($synonym['name'], 'lower');
  $name = affiliate_store_convert_term_case($name);
  db_query(
    "UPDATE {term_synonym} SET name = '%s' WHERE tid = %d",
    $name, $synonym['tid']
  );
  array_shift($context['sandbox']['data']);

  $context['results'][] = t(
    'Updated term synonym %name', array('%name' => $name)
  );
  ++$context['sandbox']['progress'];

  $context['finished'] = (
    $context['sandbox']['data'] ?
    ($context['sandbox']['progress'] / $context['sandbox']['total']) : 1
  );
}

/**
 * Batch finish callback for term renaming.
 */
function affiliate_store_batch_rename_terms_finish($success, $results,
$operations) {
  if ($success) {
    $count = count($results);
    if ($count) {
      $message = format_plural(
        $count,
        '1 term has been renamed.',
        '@count terms have been renamed.'
      );
      watchdog(
        'affiliate store',
        'Existing terms renamed. !result',
        array('!result' => theme('item_list', $results))
      );
    }
    else {
      $message = t('No term has been renamed.');
    }
  }
  else {
    $operation = reset($operations);
    $message = t(
      'An error occurred while processing %op with arguments: @args',
      array('%op' => $operation[0], '@args' => print_r($operation[1], TRUE))
    );
  }
  drupal_set_message($message, $success ? 'status' : 'error');
}
