<?php

/**
 * @file
 * Admin pages.
 */

/**
 * Menu callback; Return a page for affiliate store settings.
 */
function affiliate_store_settings_page(&$form_state) {
  // Show affiliate store setup error if any.
  if (empty($form_state['post'])) {
    affiliate_store_include('common');
    foreach (affiliate_store_setup_error() as $error) {
      drupal_set_message($error, 'error');
    }
  }

  $form['affiliate_store_title'] = array(
    '#type' => 'textfield',
    '#title' => t('Store title'),
    '#description' => t(
      'The title will be used in block, menu, and ' .
      '<a href="@storefront">storefront</a>.',
      array('@storefront' => url('affiliate-store'))
    ),
    '#default_value' => affiliate_store_title(t('Store')),
    '#required' => TRUE,
  );
  $form['affiliate_store_domain'] = array(
    '#type' => 'item',
    '#title' => t('Store domain'),
    '#description' => t(
      'This is your website domain you should use to associate with your ' .
      'affiliate store account on the Hub.'
    ),
    '#value' => check_url(affiliate_store_id()),
  );
  $form['affiliate_store_api_key'] = array(
    '#type' => 'textfield',
    '#title' => t('Service API key'),
    '#description' => t(
      'Enter the service API key you obtained from !hub. You can sign up for ' .
      'one at !url if you are not already a member.',
      array(
        '!hub' => AFFILIATE_STORE_HUB,
        '!url' => l(
          AFFILIATE_STORE_HUB, AFFILIATE_STORE_HUB_URL,
          array('attributes' => array('target' => '_blank'))
        ),
      )
    ),
    '#default_value' => variable_get('affiliate_store_api_key', ''),
    '#required' => TRUE,
  );
  $form['affiliate_store_message'] = array(
    '#type' => 'textarea',
    '#title' => t('Storefront message'),
    '#description' => t(
      'Message to display at the storefront, useful for showing disclaimer, ' .
      'recommendation, or instruction for your users.'
    ),
    '#default_value' => variable_get('affiliate_store_message', ''),
  );

  $form = system_settings_form($form);
  // Custom reset callback to prevent important settings from being reset.
  $form['buttons']['reset']['#submit'] = array(
    'affiliate_store_settings_page_reset',
    'system_settings_form_submit',
  );
  return $form;
}

/**
 * Reset callback for affiliate_store_settings_page().
 *
 * This custom reset callback prevents important settings from being reset.
 */
function affiliate_store_settings_page_reset($form, &$form_state) {
  // Prevent service API key from being reset as it can be lost forever if user
  // doesn't keep it anywhere else.
  unset($form_state['values']['affiliate_store_api_key']);
}

/**
 * Menu callback; Return a page for affiliate store advanced settings.
 */
function affiliate_store_advanced_settings_page(&$form_state) {
  // Return bulk operations page if bulk operations button has been pressed.
  if (isset($form_state['values']['op'])
    && ($form_state['values']['op'] === t('Rename existing terms'))
  ) {
    return affiliate_store_term_bulk_rename_page($form_state);
  }

  $form['cache'] = array(
    '#type' => 'fieldset',
    '#title' => t('Clear cached data'),
    '#description' => t(
      "Clear affiliate store caches. Only needed if your website has caching " .
      "turned on and you want to refresh stale pages in affiliate store " .
      "after making changes, e.g. in category mapping."
    ),
  );
  $form['cache']['clear'] = array(
    '#type' => 'submit',
    '#value' => t('Clear cached data'),
    '#submit' => array('affiliate_store_settings_cache_submit'),
  );

  $form['product'] = array(
    '#type' => 'fieldset',
    '#title' => t('Product'),
    '#description' => t('Configure product display options.'),
  );
  $form['product']['affiliate_store_product_call_to_action'] = array(
    '#type' => 'textfield',
    '#title' => t('Call to action text'),
    '#description' => t(
      'The text to use for persuading user to click on the product link.'
    ),
    '#default_value' => variable_get(
      'affiliate_store_product_call_to_action', t('Buy now')
    ),
    '#required' => TRUE,
  );
  $form['product']['affiliate_store_product_use_general'] = array(
    '#type' => 'checkbox',
    '#title' => t('Use pseudo general categories'),
    '#description' => t(
      "Check to use term 'general' in place of empty categories, when they " .
      "are not assigned by merchant for a product."
    ),
    '#default_value' => variable_get(
      'affiliate_store_product_use_general', FALSE
    ),
  );
  $options = array();
  $vocabs = taxonomy_get_vocabularies('affiliate_store_product');
  foreach ($vocabs as $vocab) {
    $options[$vocab->vid] = check_plain($vocab->name);
  }
  $form['product']['affiliate_store_product_hide_vocabularies'] = array(
    '#type' => 'checkboxes',
    '#title' => t('Hide vocabularies'),
    '#description' => t(
      'Choose the vocabularies whose terms should not appear when viewing ' .
      'product.'
    ),
    '#options' => $options,
    '#default_value' => array_keys(
      array_filter(
        variable_get(
          'affiliate_store_product_hide_vocabularies',
          array(
            affiliate_store_get_vocabulary_id('network') => 1,
            affiliate_store_get_vocabulary_id('merchant') => 1,
          )
        )
      )
    ),
  );

  $form['update'] = array(
    '#type' => 'fieldset',
    '#title' => t('Update'),
    '#description' => t('Configure affiliate store update behaviour.'),
  );
  $form['update']['affiliate_store_auto_update'] = array(
    '#type' => 'checkbox',
    '#title' => t('Update affiliate store automatically'),
    '#description' => t('Uncheck to disable automatic update via cron.'),
    '#default_value' => variable_get('affiliate_store_auto_update', TRUE),
  );
  $form['update']['affiliate_store_retrieve_limit'] = array(
    '#type' => 'select',
    '#title' => t('Retrieval limit'),
    '#description' => t(
      'Maximum number of items to retrieve in one service call to the Hub. ' .
      'Larger value makes update faster but uses more bandwidth and memory. ' .
      'Lowering this value may help with update problem.'
    ),
    '#options' => drupal_map_assoc(
      array(
        10, 20, 30, 40, 50, 60, 70, 80, 90,
        100, 200, 300, 400, 500, 600, 700, 800, 900, 1000,
      )
    ),
    '#default_value' => variable_get('affiliate_store_retrieve_limit', 500),
  );
  $form['update']['affiliate_store_update_throttle'] = array(
    '#type' => 'select',
    '#title' => t('Update throttle'),
    '#description' => t(
      'Maximum number of operations to perform in one pass of update process ' .
      'before waiting for next turn. Larger value makes update faster but ' .
      'uses more system resources.'
    ),
    '#options' => drupal_map_assoc(
      array(
        10, 20, 30, 40, 50, 60, 70, 80, 90,
        100, 200, 300, 400, 500, 600, 700, 800, 900, 1000,
      )
    ),
    '#default_value' => variable_get('affiliate_store_update_throttle', 50),
  );

  $form['taxonomy'] = array(
    '#type' => 'fieldset',
    '#title' => t('Taxonomy'),
    '#description' => t(
      'Configure how taxonomy terms are imported into affiliate store.'
    ),
  );
  $form['taxonomy']['affiliate_store_term_case'] = array(
    '#type' => 'radios',
    '#title' => t('Term name case conversion'),
    '#description' => t(
      'Case conversion to use for category terms. Smartcase is like ' .
      'titlecase but with words designated as stop words below in lowercase.'
    ),
    '#options' => array(
      'none' => 'No conversion',
      'lower' => 'Lowercase: this is an example',
      'upper' => 'Uppercase: THIS IS AN EXAMPLE',
      'title' => 'Titlecase: This Is An Example',
      'smart' => 'Smartcase: This is an Example',
    ),
    '#default_value' => variable_get('affiliate_store_term_case', 'smart'),
    '#required' => TRUE,
  );
  $form['taxonomy']['affiliate_store_smartcase_stop_words'] = array(
    '#type' => 'textarea',
    '#title' => t('Smartcase stop words'),
    '#description' => t(
      'Words that should remain lowercase when not used as first word in a ' .
      'phrase, separated by comma. Do not use WYSIWYG editors on this field.'
    ),
    '#default_value' => variable_get(
      'affiliate_store_smartcase_stop_words',
      'a,an,and,as,at,by,for,is,in,into,of,off,on,onto,or,per,the,to,with'
    ),
  );
  $form['taxonomy']['submit'] = array(
    '#type' => 'submit',
    '#value' => t('Rename existing terms'),
    '#submit' => array(
      // Save settings before renaming terms.
      'affiliate_store_advanced_settings_page_submit',
      'system_settings_form_submit',
      'affiliate_store_settings_taxonomy_submit',
    ),
  );

  $form['#submit'] = array('affiliate_store_advanced_settings_page_submit');
  return system_settings_form($form);
}

/**
 * Submit callback for affiliate_store_advanced_settings_page().
 */
function affiliate_store_advanced_settings_page_submit($form, &$form_state) {
  // Trim spaces between stop words.
  $stopwords = &$form_state['values']['affiliate_store_smartcase_stop_words'];
  $stopwords = implode(',', array_map('trim', explode(',', $stopwords)));
}

/**
 * Submit callback for affiliate_store_advanced_settings_page() cache settings.
 */
function affiliate_store_settings_cache_submit($form, &$form_state) {
  affiliate_store_include('common');
  affiliate_store_clear_cache();
  drupal_set_message(t('Affiliate store caches have been cleared.'));
}

/**
 * Submit callback for affiliate_store_advanced_settings_page() taxonomy
 * settings.
 */
function affiliate_store_settings_taxonomy_submit($form, &$form_state) {
  // Rebuild form to go to 2nd step, e.g. to show confirmation form.
  $form_state['rebuild'] = TRUE;
  // This is needed because it had been unset after settings saved by
  // system_settings_form_submit().
  $form_state['values']['op'] = t('Rename existing terms');
}

/**
 * Menu callback; Return category mapping page.
 *
 * @param string $vocabulary
 *   Term vocabulary.
 */
function affiliate_store_category_map_page(&$form_state, $vocabulary) {
  // Return bulk operations page if bulk operations button has been pressed.
  if (isset($form_state['values']['op'])) {
    switch ($form_state['values']['op']) {
      case t('Delete'):
        return affiliate_store_term_synonym_bulk_delete_page(
          $form_state,
          array_keys(array_filter($form_state['values']['checkboxes']))
        );
      case t('Merge'):
        return affiliate_store_term_synonym_bulk_merge_page(
          $form_state,
          array_keys(array_filter($form_state['values']['checkboxes']))
        );
    }
  }

  $form = array();
  $destination = drupal_get_destination();
  $miss_tid = affiliate_store_get_term_id('', $vocabulary);
  $header = array(
    array('data' => t('Term'), 'field' => 'td.name', 'sort' => 'asc'),
    array('data' => t('Synonym'), 'field' => 'ts.name'),
  );

  $result = pager_query(
    "SELECT td.tid, td.vid, td.name, ts.name AS synonym
    FROM {term_data} AS td
    LEFT JOIN {term_synonym} AS ts ON ts.tid = td.tid
    WHERE td.vid = %d" . tablesort_sql($header),
    50, 0, "SELECT COUNT(*) FROM {term_data} WHERE vid = %d",
    affiliate_store_get_vocabulary_id($vocabulary)
  );
  $checkboxes = array();
  $terms = array();
  while ($term = db_fetch_object($result)) {
    // Only the 1st synonym defined will be used.
    if (isset($checkboxes[$term->tid])) {
      continue;
    }
    // Hide pseudo general categories.
    if ($term->tid == $miss_tid) {
      continue;
    }

    $terms[$term->tid] = $term;
    $checkboxes[$term->tid] = '';
    $form['term'][$term->tid] = array(
      '#value' => check_plain($term->name),
    );
    $form['synonym'][$term->tid] = array(
      '#value' => check_plain($term->synonym),
    );
    $form['operations'][$term->tid]['edit'] = array(
      '#value' => l(
        t('edit'),
        "admin/build/affiliate-store/map/term/$term->tid/edit",
        array('query' => $destination)
      ),
    );
  }
  $form['pager'] = array('#value' => theme('pager', NULL, 50));

  $form['checkboxes'] = array(
    '#type' => 'checkboxes',
    '#options' => $checkboxes,
  );
  $form['terms'] = array(
    '#type' => 'value',
    '#value' => $terms,
  );

  $form['buttons']['merge'] = array(
    '#type' => 'submit',
    '#value' => t('Merge'),
    '#validate' => array(
      'affiliate_store_category_map_bulk_operations_validate',
    ),
    '#submit' => array('affiliate_store_category_map_bulk_operations_submit'),
  );
  $form['buttons']['delete'] = array(
    '#type' => 'submit',
    '#value' => t('Delete'),
    '#validate' => array(
      'affiliate_store_category_map_bulk_operations_validate',
    ),
    '#submit' => array('affiliate_store_category_map_bulk_operations_submit'),
  );
  return $form;
}

/**
 * Validate callback for affiliate_store_category_map_page().
 */
function affiliate_store_category_map_bulk_operations_validate($form,
&$form_state) {
  if (!count(array_filter($form_state['values']['checkboxes']))) {
    form_set_error('checkboxes', t('No terms selected.'));
  }
}

/**
 * Submit callback for affiliate_store_category_map_page().
 */
function affiliate_store_category_map_bulk_operations_submit($form,
&$form_state) {
  // Rebuild form to go to 2nd step, e.g. to show confirmation form.
  $form_state['rebuild'] = TRUE;
}

/**
 * Return term synonym edit page.
 *
 * @param int $tid
 *   Term ID whose synonym to edit.
 */
function affiliate_store_category_map_edit_page(&$form_state, $tid) {
  $term = taxonomy_get_term($tid);
  $vocab = $term ? affiliate_store_get_vocabulary_name($term->vid) : '';
  // Make sure the term exists and is managed by affiliate store. The form can
  // only save one synonym and so can delete away all synonyms of term not
  // managed by affiliate store.
  $categories = array('category', 'subcategory', 'subsubcategory');
  if (in_array($vocab, $categories, TRUE)) {
    // Cancel link parameters. Use supplied destination if exists.
    $url = array('path' => "admin/build/affiliate-store/map/$vocab");
    if (isset($_GET['destination'])) {
      $url = array_merge($url, parse_url(urldecode($_GET['destination'])));
    }

    $form['tid'] = array(
      '#type' => 'value',
      '#value' => $tid,
    );
    $form['term'] = array(
      '#type' => 'item',
      '#title' => t('Term'),
      '#value' => check_plain($term->name),
    );
    $form['synonym'] = array(
      '#type' => 'textfield',
      '#title' => t('Synonym'),
      '#default_value' => affiliate_store_get_term_synonym($tid),
    );
    $form['submit'] = array(
      '#type' => 'submit',
      '#value' => t('Save'),
    );
    $form['cancel'] = array(
      '#value' => l(t('Cancel'), $url['path'], $url),
    );
  }
  else {
    $form['error'] = array(
      '#value' => t('Term not exists or not managed by affiliate store.'),
    );
  }
  return $form;
}

/**
 * Submit callback for affiliate_store_category_map_edit_page().
 */
function affiliate_store_category_map_edit_page_submit($form, &$form_state) {
  $tid = $form_state['values']['tid'];
  $synonym = $form_state['values']['synonym'];
  db_query("DELETE FROM {term_synonym} WHERE tid = %d", $tid);
  if ($synonym) {
    affiliate_store_include('common');
    db_query(
      "INSERT INTO {term_synonym} SET tid = %d, name = '%s'",
      $tid, affiliate_store_convert_term_case($synonym)
    );
  }
}

/**
 * Return a confirmation page for bulk term renaming.
 */
function affiliate_store_term_bulk_rename_page(&$form_state) {
  $form['#submit'] = array('affiliate_store_term_bulk_rename_page_submit');
  return confirm_form(
    $form,
    t('Are you sure you want to rename existing terms?'),
    "admin/build/affiliate-store",
    t('Any prior customization will be lost.'),
    t('Rename all'),
    t('Cancel')
  );
}

/**
 * Submit callback for affiliate_store_term_bulk_rename_page().
 */
function affiliate_store_term_bulk_rename_page_submit($form, &$form_state) {
  $path = drupal_get_path('module', 'affiliate_store');
  $batch = array(
    'operations' => array(
      array('affiliate_store_batch_rename_term_names', array()),
      array('affiliate_store_batch_rename_term_synonyms', array()),
    ),
    'finished' => 'affiliate_store_batch_rename_terms_finish',
    'title' => t('Rename terms'),
    'init_message' => t('Term renaming is starting.'),
    'progress_message' => t('Finished @current out of @total.'),
    'error_message' => t('Term renaming has encountered an error.'),
    'file' => "$path/includes/admin.batch.inc",
  );
  batch_set($batch);
}

/**
 * Return a confirmation page for bulk term synonym deletion.
 *
 * @param array $terms
 *   Term ID of synonyms to delete.
 */
function affiliate_store_term_synonym_bulk_delete_page(&$form_state, $terms) {
  $items = array();
  $terms_info = array();
  foreach ($terms as $tid) {
    $terms_info[$tid] = $form_state['values']['terms'][$tid];
  }
  foreach ($terms_info as $info) {
    $items[] = t(
      'Term: %term, Synonym: %synonym',
      array('%term' => $info->name, '%synonym' => $info->synonym)
    );
  }

  $form['terms'] = array(
    '#type' => 'value',
    '#value' => $terms,
  );
  $form['term_list'] = array(
    '#value' => theme('item_list', $items),
  );
  $form['#submit'] = array(
    'affiliate_store_term_synonym_bulk_delete_page_submit',
  );
  $vocab = affiliate_store_get_vocabulary_name($terms_info[$terms[0]]->vid);
  return confirm_form(
    $form,
    t('Are you sure you want to delete these term synonyms?'),
    "admin/build/affiliate-store/map/$vocab",
    t('This action cannot be undone.'),
    t('Delete all'),
    t('Cancel')
  );
}

/**
 * Submit callback for affiliate_store_term_synonym_bulk_delete_page().
 */
function affiliate_store_term_synonym_bulk_delete_page_submit($form,
&$form_state) {
  $terms = $form_state['values']['terms'];
  db_query(
    "DELETE FROM {term_synonym}
    WHERE tid IN (" . db_placeholders($terms, 'int') . ")",
    $terms
  );
  drupal_set_message(
    format_plural(
      count($terms),
      'The term synonym has been deleted.',
      '@count term synonyms have been deleted.'
    )
  );
}

/**
 * Return a confirmation page for bulk term synonym merging.
 *
 * @param array $terms
 *   Term ID of synonyms to merge.
 */
function affiliate_store_term_synonym_bulk_merge_page(&$form_state, $terms) {
  $default_synonym = '';
  $items = array();
  $terms_info = array();
  foreach ($terms as $tid) {
    $terms_info[$tid] = $form_state['values']['terms'][$tid];
  }
  foreach ($terms_info as $info) {
    // Use the 1st non-empty synonym as default.
    if (!$default_synonym) {
      $default_synonym = $info->synonym;
    }
    $items[] = t(
      'Term: %term, Synonym: %synonym',
      array('%term' => $info->name, '%synonym' => $info->synonym)
    );
  }

  $form['terms'] = array(
    '#type' => 'value',
    '#value' => $terms,
  );
  $form['term_list'] = array(
    '#value' => theme('item_list', $items),
  );
  $form['synonym'] = array(
    '#type' => 'textfield',
    '#title' => t('Synonym'),
    '#description' => t('Synonym for all the selected terms.'),
    '#default_value' => $default_synonym,
  );
  $form['#validate'] = array(
    'affiliate_store_term_synonym_bulk_merge_page_validate',
  );
  $form['#submit'] = array(
    'affiliate_store_term_synonym_bulk_merge_page_submit',
  );
  $vocab = affiliate_store_get_vocabulary_name($terms_info[$terms[0]]->vid);
  return confirm_form(
    $form,
    t("Merge categories"),
    "admin/build/affiliate-store/map/$vocab",
    '',
    t('Save'),
    t('Cancel')
  );
}

/**
 * Validate callback for affiliate_store_term_synonym_bulk_merge_page().
 */
function affiliate_store_term_synonym_bulk_merge_page_validate($form,
&$form_state) {
  if (!$form_state['values']['synonym']) {
    form_set_error('synonym', t('You must specify a synonym.'));
  }
}

/**
 * Submit callback for affiliate_store_term_synonym_bulk_merge_page().
 */
function affiliate_store_term_synonym_bulk_merge_page_submit($form,
&$form_state) {
  $terms = $form_state['values']['terms'];
  $synonym = $form_state['values']['synonym'];
  // Only make changes if synonym not empty to prevent term synonyms from
  // being deleted.
  if ($synonym) {
    affiliate_store_include('common');
    $synonym = affiliate_store_convert_term_case($synonym);
    db_query(
      "DELETE FROM {term_synonym}
      WHERE tid IN (" . db_placeholders($terms, 'int') . ")",
      $terms
    );
    foreach ($terms as $tid) {
      db_query(
        "INSERT INTO {term_synonym} SET tid = %d, name = '%s'", $tid, $synonym
      );
    }
    drupal_set_message(
      format_plural(
        count($terms),
        'The term synonym has been merged.',
        '@count term synonyms have been merged.'
      )
    );
  }
}

/**
 * Menu callback; Return affiliate store subscriptions page.
 */
function affiliate_store_subscription_page(&$form_state) {
  // Return bulk operations page if bulk operations button has been pressed.
  if (isset($form_state['values']['op'])
    && ($form_state['values']['op'] === t('Refetch'))
  ) {
    return affiliate_store_subscription_bulk_refetch_page(
      $form_state, array_keys(array_filter($form_state['values']['checkboxes']))
    );
  }

  $form = array();
  $categories = array('category', 'subcategory', 'subsubcategory');
  $categories = affiliate_store_vocabularies($categories);
  foreach (array_keys($categories) as $vocab) {
    $categories[$vocab]['miss_tid'] = affiliate_store_get_term_id('', $vocab);
  }
  $header = array(
    array('data' => t('Network'), 'field' => 'network', 'sort' => 'asc'),
    array('data' => t('Merchant'), 'field' => 'merchant'),
    array('data' => t('Category'), 'field' => 'category'),
    array('data' => t('Subcategory'), 'field' => 'subcategory'),
    array('data' => t('Subsubcategory'), 'field' => 'subsubcategory'),
    array('data' => t('Product'), 'field' => 'product'),
  );

  $on = array();
  foreach (affiliate_store_vocabularies() as $vocab => $info) {
    $on[] = "p.{$info['as']}tid = ssn.{$info['as']}tid";
  }
  $result = pager_query(
    "SELECT ssn.ssnid, ssn.ctid, ssn.sctid, ssn.ssctid,
            t1.name AS network,
            t2.name AS merchant,
            t3.name AS category,
            t4.name AS subcategory,
            t5.name AS subsubcategory,
            COUNT(DISTINCT p.nid) AS product
    FROM {affiliate_store_subscription} AS ssn
    LEFT JOIN {term_data} AS t1 ON t1.tid = ssn.nwtid
    LEFT JOIN {term_data} AS t2 ON t2.tid = ssn.mtid
    LEFT JOIN {term_data} AS t3 ON t3.tid = ssn.ctid
    LEFT JOIN {term_data} AS t4 ON t4.tid = ssn.sctid
    LEFT JOIN {term_data} AS t5 ON t5.tid = ssn.ssctid
    LEFT JOIN {affiliate_store_product} AS p ON " . implode(' AND ', $on) . "
    WHERE ssn.refetch = %d
    GROUP BY ssn.ssnid" .
    tablesort_sql($header),
    50, 0, "SELECT COUNT(*) FROM {affiliate_store_subscription}", 0
  );
  $checkboxes = array();
  $subscriptions = array();
  while ($subscription = db_fetch_object($result)) {
    // Hide pseudo general categories.
    foreach ($categories as $vocab => $info) {
      if ($subscription->{$info['as'] . 'tid'} == $info['miss_tid']) {
        $subscription->{$vocab} = '';
      }
    }

    $subscriptions[$subscription->ssnid] = $subscription;
    $checkboxes[$subscription->ssnid] = '';
    $form['network'][$subscription->ssnid] = array(
      '#value' => check_plain($subscription->network),
    );
    $form['merchant'][$subscription->ssnid] = array(
      '#value' => check_plain($subscription->merchant),
    );
    $form['category'][$subscription->ssnid] = array(
      '#value' => check_plain($subscription->category),
    );
    $form['subcategory'][$subscription->ssnid] = array(
      '#value' => check_plain($subscription->subcategory),
    );
    $form['subsubcategory'][$subscription->ssnid] = array(
      '#value' => check_plain($subscription->subsubcategory),
    );
    $form['product'][$subscription->ssnid] = array(
      '#value' => $subscription->product,
    );
  }
  $form['pager'] = array('#value' => theme('pager', NULL, 50));

  $form['checkboxes'] = array(
    '#type' => 'checkboxes',
    '#options' => $checkboxes,
  );
  $form['subscriptions'] = array(
    '#type' => 'value',
    '#value' => $subscriptions,
  );

  $form['submit'] = array(
    '#type' => 'submit',
    '#value' => t('Refetch'),
    '#validate' => array(
      'affiliate_store_subscription_bulk_operations_validate',
    ),
    '#submit' => array('affiliate_store_subscription_bulk_operations_submit'),
  );
  return $form;
}

/**
 * Validate callback for affiliate_store_subscription_page().
 */
function affiliate_store_subscription_bulk_operations_validate($form,
&$form_state) {
  if (!count(array_filter($form_state['values']['checkboxes']))) {
    form_set_error('checkboxes', t('No subscriptions selected.'));
  }
}

/**
 * Submit callback for affiliate_store_subscription_page().
 */
function affiliate_store_subscription_bulk_operations_submit($form,
&$form_state) {
  // Rebuild form to go to 2nd step, e.g. to show confirmation form.
  $form_state['rebuild'] = TRUE;
}

/**
 * Return a confirmation page for bulk subscriptions refetch.
 *
 * @param array $subscriptions
 *   Subscription IDs to refetch.
 */
function affiliate_store_subscription_bulk_refetch_page(&$form_state,
$subscriptions) {
  $form['subscriptions'] = array(
    '#type' => 'value',
    '#value' => $subscriptions,
  );
  $items = array();
  $subscriptions_info = array();
  foreach ($subscriptions as $ssnid) {
    $subscriptions_info[$ssnid] = (
      $form_state['values']['subscriptions'][$ssnid]
    );
  }
  foreach ($subscriptions_info as $info) {
    $items[] = t(
      'Network: %network, Merchant: %merchant, Category: %category, ' .
      'Subcategory: %subcategory, Subsubcategory: %subsubcategory',
      array(
        '%network' => $info->network,
        '%merchant' => $info->merchant,
        '%category' => $info->category,
        '%subcategory' => $info->subcategory,
        '%subsubcategory' => $info->subsubcategory,
      )
    );
  }
  $form['subscription_list'] = array(
    '#value' => theme('item_list', $items),
  );
  $form['#submit'] = array(
    'affiliate_store_subscription_bulk_refetch_page_submit',
  );
  return confirm_form(
    $form,
    t('Are you sure you want to refetch the products of these subscriptions?'),
    'admin/build/affiliate-store/subscriptions',
    t('This action cannot be undone.'),
    t('Refetch all'),
    t('Cancel')
  );
}

/**
 * Submit callback for affiliate_store_subscription_bulk_refetch_page().
 */
function affiliate_store_subscription_bulk_refetch_page_submit($form,
&$form_state) {
  $subscriptions = $form_state['values']['subscriptions'];
  affiliate_store_include('subscription');
  affiliate_store_subscription_refetch($subscriptions);
  drupal_set_message(
    format_plural(
      count($subscriptions),
      'The subscription will be refetched at the next update.',
      '@count subscriptions will be refetched at the next update.'
    )
  );
}

/**
 * Menu callback; Return affiliate store update page.
 */
function affiliate_store_update_page(&$form_state) {
  $form['submit'] = array(
    '#type' => 'submit',
    '#value' => t('Update'),
  );

  affiliate_store_include('common');
  if (affiliate_store_setup_error()) {
    $form['submit']['#disabled'] = TRUE;
    $message = t(
      'There is problem with your affiliate store setup. Please check your ' .
      '<a href="@settings">affiliate store settings</a>.',
      array('@settings' => url('admin/build/affiliate-store'))
    );
    drupal_set_message($message, 'error');
  }

  return $form;
}

/**
 * Submit callback for affiliate_store_update_page().
 */
function affiliate_store_update_page_submit($form, &$form_state) {
  $options = array(
    'update_throttle' => variable_get('affiliate_store_update_throttle', 50),
    'sleep' => 1,
  );
  $path = drupal_get_path('module', 'affiliate_store');
  $batch = array(
    'operations' => array(
      array(
        'affiliate_store_batch_import_subscription',
        array(t('Importing pending data: @current of @total.'), $options),
      ),
      array('affiliate_store_batch_fetch_subscription', array($options)),
      array(
        'affiliate_store_batch_import_subscription',
        array(t('Importing new data: @current of @total.'), $options),
      ),
    ),
    'finished' => 'affiliate_store_batch_update_finish',
    'title' => t('Updating affiliate store'),
    'init_message' => t('Update process is starting.'),
    'progress_message' => t('Finished @current out of @total.'),
    'error_message' => t('Update has encountered an error.'),
    'file' => "$path/includes/admin.batch.inc",
  );
  batch_set($batch);
}
