<?php

/**
 * @file
 * Theme and template preprocess functions.
 */

/**
 * Preprocess affiliate store product page.
 *
 * @see affiliate-store.tpl.php
 */
function template_preprocess_affiliate_store(&$variables) {
  affiliate_store_add_css('affiliate_store');
  $variables['template_files'][] = 'affiliate-store';

  $variables['message'] = variable_get('affiliate_store_message', '');
  $variables['message'] = filter_xss_admin($variables['message']);
  $variables['trail'] = theme('affiliate_store_trail', $variables['trail']);
  // Only display browse fieldset if there is browse item.
  $variables['browse'] = '';
  if ($variables['browse_items']) {
    $browse = array(
      '#attributes' => array(),
      '#title' => $variables['browse_title'],
      '#value' => theme('item_list', $variables['browse_items']),
    );
    $variables['browse'] = theme('fieldset', $browse);
  }

  // Term information of currently displayed product category.
  $term = $variables['term'];
  $variables['term_name'] = $term ? check_plain($term->name) : '';
  $variables['tid'] = $term ? $term->tid : 0;
  $variables['vid'] = $term ? $term->vid : 0;
}

/**
 * Preprocess affiliate store views.
 *
 * @see views-view--affiliate-store.tpl.php
 */
function template_preprocess_views_view__affiliate_store(&$view) {
  affiliate_store_add_css('views_affiliate_store');
}

/**
 * Output currency symbol from currency code.
 *
 * @param string $code
 *   Standard 3-letter currency code.
 *
 * @return string
 *   Currency symbol, or uppercased currency code if no symbol found.
 *
 * @ingroup themable
 */
function theme_affiliate_store_currency($code) {
  $currency_symbols = array(
    'CHF' => 'Fr',
    'CNY' => '元',
    'EUR' => '€',
    'GBP' => '£',
    'JPY' => '¥',
    'NOK' => 'Kr',
    'USD' => '$',
  );
  $code = drupal_strtoupper(check_plain($code));
  return !empty($currency_symbols[$code]) ? $currency_symbols[$code] : $code;
}

/**
 * Output product link.
 *
 * @param string $text
 *   Link anchor text.
 * @param string $url
 *   Product URL.
 *
 * @return string
 *   Product link HTML.
 */
function theme_affiliate_store_go($text, $url) {
  $attributes = array(
    'class' => 'affiliate-store-go',
    'rel' => 'nofollow',
    'target' => '_blank',
  );
  return l($text, $url, array('attributes' => $attributes));
}

/**
 * Output menu tree.
 *
 * @param array $tree
 *   An array of menu items, each menu item is an associative array with the
 *   following keys:
 *   - name: Name of menu item.
 *   - description: (optional) Menu item description, will be used as link
 *     title attribute if path is specified.
 *   - path: (optional) Menu item internal path or external URL, will be output
 *     as link.
 *   - active: (optional) TRUE to signify the menu item is currently active or
 *     selected.
 *   - children: (optional) Nested menu items if any.
 *
 * @return string
 *   Menu tree HTML.
 */
function theme_affiliate_store_menu($tree) {
  $output = '<ul class="menu">';
  $keys = array_keys($tree);
  for ($i = 0, $count = count($keys); $i < $count; ++$i) {
    $value = $tree[$keys[$i]];
    $classes = array();

    if (!empty($value['children'])) {
      $classes[] = !empty($value['active']) ? 'expanded' : 'collapsed';
    }
    else {
      $classes[] = 'leaf';
    }

    if ($i == 0) {
      $classes[] = 'first';
    }
    elseif ($i == ($count - 1)) {
      $classes[] = 'last';
    }

    if (!empty($value['active'])) {
      $classes[] = 'active-trail';
    }

    $output .= '<li class="' . implode(' ', $classes) . '">';
    if (!empty($value['path'])) {
      $options = array();
      if (!empty($value['description'])) {
        $options['attributes']['title'] = check_plain($value['description']);
      }
      $output .= l($value['name'], $value['path'], $options);
    }
    else {
      $output .= check_plain($value['name']);
    }

    if (!empty($value['children']) && !empty($value['active'])) {
      $output .= theme_affiliate_store_menu($value['children']);
    }
    $output .= '</li>';
  }
  $output .= '</ul>';
  return $output;
}

/**
 * Output affiliate store selection trail.
 *
 * @param mixed $trail
 *   A single trail link or an array of trail links.
 * @param string $separator
 *   (optional) Separator between trail links. Default to '»'.
 *
 * @return string
 *   Trail HTML.
 */
function theme_affiliate_store_trail($trail, $separator = '»') {
  $trail = is_array($trail) ? implode(" $separator ", $trail) : $trail;
  return "<div class='affiliate-store-trail'>$trail</div>";
}
